#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Config {
    pub anti_phishing_code: String,

    pub keymap: Option<String>,
    pub modules: Option<String>,

    pub auths: Vec<Auth>,

    pub networks: Vec<Network>,

    #[serde(default)]
    pub ssh: SSHServer,

    #[serde(default)]
    pub pre_lvm_crypt: Vec<CryptDev>,
    #[serde(default)]
    pub lvm: Vec<LvmVG>,
    #[serde(default)]
    pub crypt: Vec<CryptDev>,

    pub bootstrap: Bootstrap,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Auth {
    pub name: String,
    #[serde(rename = "sshKey")]
    pub ssh_key: String,
    pub password: String,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Network {
    pub name: String,
    pub interfaces: Vec<NetworkInterface>,
    pub script: String,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct NetworkInterface {
    pub var: String,
    pub n: usize,
    pub regexps: Vec<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct SSHServer {
    pub listen: String,
    pub keys: SSHKeys,
}
impl Default for SSHServer {
    fn default() -> Self {
        Self {
            listen: "[::]:22".to_string(),
            keys: SSHKeys::default(),
        }
    }
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct SSHKeys {
    dsa: Option<String>,
    rsa: Option<String>,
    ecdsa: Option<String>,
    ed25519: Option<String>,
}
impl SSHKeys {
    pub fn iter(&self) -> impl Iterator<Item = &str> {
        [
            self.dsa.iter(),
            self.rsa.iter(),
            self.ecdsa.iter(),
            self.ed25519.iter(),
        ]
        .into_iter()
        .flatten()
        .map(String::as_str)
    }
}
impl Default for SSHKeys {
    fn default() -> Self {
        Self {
            dsa: Some("id_dsa".to_string()),
            rsa: Some("id_rsa".to_string()),
            ecdsa: Some("id_ecdsa".to_string()),
            ed25519: Some("id_ed25519".to_string()),
        }
    }
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmVG {
    vg: String,
    pvs: LvmPV,

    #[serde(default)]
    defaults: LvmLVDefaults,

    lvs: Vec<LvmLV>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmLVDefaults {
    #[serde(default = "default_fs")]
    fs: String,
    raid: Option<Raid>,
}
impl Default for LvmLVDefaults {
    fn default() -> Self {
        Self {
            fs: default_fs(),
            raid: None,
        }
    }
}

fn default_fs() -> String {
    "ext4".to_string()
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmLV {
    name: String,
    #[serde(skip_serializing_if = "Option::is_none")]
    fs: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    raid: Option<Raid>,
    #[serde(skip_serializing_if = "Option::is_none")]
    size: Option<String>,
    #[serde(skip_serializing_if = "Option::is_none")]
    extents: Option<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct LvmPV {
    n: i16,
    regexps: Vec<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct CryptDev {
    name: String,
    dev: Option<String>,
    prefix: Option<String>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Raid {
    mirrors: Option<u8>,
    stripes: Option<u8>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
pub struct Bootstrap {
    dev: String,
    seed: String,
}
