use std::io;
use std::process::Command;

#[derive(Debug, serde::Deserialize, serde::Serialize)]
struct Report {
    report: Vec<ReportObj>,
}

#[derive(Debug, serde::Deserialize, serde::Serialize)]
#[serde(untagged)]
enum ReportObj {
    PV { pv: Vec<PV> },
    VG { vg: Vec<VG> },
    LV { lv: Vec<LV> },
}

#[derive(Debug, Clone, serde::Deserialize, serde::Serialize)]
pub struct PV {
    pub pv_name: String,
    pub vg_name: String,
    pub pv_fmt: String,
    pub pv_attr: String,
    pub pv_size: String,
    pub pv_free: String,
}

#[derive(Debug, Clone, serde::Deserialize, serde::Serialize)]
pub struct VG {
    pub vg_name: String,
    pub pv_count: String,
    pub lv_count: String,
    pub snap_count: String,
    pub vg_attr: String,
    pub vg_size: String,
    pub vg_free: String,
}

#[derive(Debug, Clone, serde::Deserialize, serde::Serialize)]
pub struct LV {
    lv_name: String,
    vg_name: String,
    lv_attr: String,
    lv_size: String,
    pool_lv: String,
    origin: String,
    data_percent: String,
    metadata_percent: String,
    move_pv: String,
    mirror_log: String,
    copy_percent: String,
    convert_lv: String,
}

pub fn pvs() -> io::Result<Vec<PV>> {
    report_cmd("pvs", |o| match o {
        ReportObj::PV { pv } => Some(pv),
        _ => None,
    })
}

pub fn vgs() -> io::Result<Vec<VG>> {
    report_cmd("vgs", |o| match o {
        ReportObj::VG { vg } => Some(vg),
        _ => None,
    })
}

pub fn lvs() -> io::Result<Vec<LV>> {
    report_cmd("lvs", |o| match o {
        ReportObj::LV { lv } => Some(lv),
        _ => None,
    })
}

fn report_cmd<T>(cmd: &str, find: fn(ReportObj) -> Option<Vec<T>>) -> io::Result<Vec<T>> {
    let output = Command::new(cmd).arg("--reportformat=json").output()?;
    let report: Report = serde_json::from_slice(output.stdout.as_slice()).unwrap();
    Ok(report
        .report
        .into_iter()
        .filter_map(find)
        .flatten()
        .collect())
}
