use eyre::Result;
use log::error;
use std::env;
use std::process::exit;

use init::cmd;

fn main() -> Result<()> {
    env_logger::builder()
        .format_timestamp_millis()
        .format_target(false)
        .filter_level(log::LevelFilter::Info)
        .write_style(env_logger::WriteStyle::Always)
        .init();

    unsafe {
        env::set_var("PATH", "/bin:/sbin:/usr/bin:/usr/sbin");
    }

    let call_name = env::args().next().unwrap_or("init".into());
    let call_name = (call_name.rsplit_once('/').map(|(_, n)| n)).unwrap_or(call_name.as_str());

    match call_name {
        "init" => cmd::init::run()?,
        "init-version" => cmd::version::run(),
        _ => {
            error!("invalid call name: {call_name:?}");
            exit(1);
        }
    };

    Ok(())
}
